/*-----------------------------------------------------------------------
  GIGNULAMBDA.C

  SYNTAX:
    (nu,lambda)=gignulambda(x,alpha,beta,tau,n)
       x is a vector of transformed signals
       alpha is a positive scalar
       beta is a positive scalar
       tau is an integer > 1
       n is an integer >= 1

  Returns nu and lambda of the same dimensions as x for the GIG model.

  Requires header file nulambda.h.
-----------------------------------------------------------------------*/
#include <math.h>
#include "mex.h"
#include "nulambda.h"
#define betafunc(a,b) exp(lgamma(a)+lgamma(b)-lgamma(a+b))

/* GIGNuLambda function */
void gignulambda(double *nuvec, double *lambdavec, double *x, double alpha, 
	      double beta, int tau,int n, int length) {
 double psinum,psiden,psiden2,*c1,*c2,*poly,const1,const2,z,
   psilimnum,psilimden,psilimden2,s;
 int i;
 c1=(double *)calloc(((n-1)*(tau-1)+1),sizeof(double));
 c2=(double *)calloc(((n-2)*(tau-1)+1),sizeof(double));
 poly=(double *)calloc(tau,sizeof(double));
 /* Define e_m(z) */
 *poly=1;
   for (i=1;i<tau;i++)
     *(poly+i)=(*(poly+i-1))/i;
 /* Define c(m,n) coefficients on e_m(z)^n */
   if ((n-1)==0)
     *c1=1;
   else
     poly_exp(c1,poly,n-1,tau-1);
   if ((n-2)==0)
     *c2=1;
   else
     poly_exp(c2,poly,n-2,tau-1);
   for (i=0;i<length;i++) {
     if ((*(x+i))!=0) {
       s=1/(*(x+i));     /* raw signal */
       psinum=psi_c(s,alpha+1,beta,tau,n-2,n,c2);
       psiden=psi_c(s,alpha,beta,tau,n-2,n,c2);
       psiden2=psi_c(s,alpha,beta,tau,n-1,n,c1);
       *(nuvec+i)=((alpha+2*tau)*psinum)/
                           ((beta+n*tau*s)*psiden);
     /* Unnecessary to calculate lambda under 2nd price rule */
       if (lambdavec!=NULL) {
         z=(tau*s)/(beta+n*tau*s);
	 *(lambdavec+i)=(s*(n-1)*pow(z,tau)*psiden)/
	                 (betafunc(alpha+tau,tau)*psiden2);
       }
     }
     /* Special case of x==0 */
     else {
       *(nuvec+i)=0;
       if (lambdavec!=NULL)
	 *(lambdavec+i)=mxGetInf();
     }
   }
}

/*--------------------------------mexFunction--------------------------------*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  double *nuvec,*lambdavec,*x,alpha,beta,t1,n1;
  int tau,n,i,length;
  if (nrhs!=5)
    mexErrMsgTxt("Error. Syntax is gignulambda(s,alpha,beta,tau,n).");
  x=mxGetPr(prhs[0]);
  length=mxGetM(prhs[0]);
  alpha=mxGetScalar(prhs[1]);
  if (alpha<=0)
    mexErrMsgTxt("Error: alpha must be positive.");
  beta=mxGetScalar(prhs[2]);
  if (beta<=0)
    mexErrMsgTxt("Error: beta must be positive.");
  tau=mxGetScalar(prhs[3])+.5;
  if (tau<1)
    mexErrMsgTxt("Error: tau must be greater than or equal to one.");
  n=mxGetScalar(prhs[4])+.5;
  if (n<1)
    mexErrMsgTxt("Error: n must be greater than or equal to one.");
  plhs[0]=mxCreateDoubleMatrix(length,1,mxREAL);
  nuvec=mxGetPr(plhs[0]);
  if (nlhs==2) {
    plhs[1]=mxCreateDoubleMatrix(length,1,mxREAL);
    lambdavec=mxGetPr(plhs[1]);
  }
  else
    lambdavec=NULL;
  gignulambda(nuvec,lambdavec,x,alpha,beta,tau,n,length);
  return;
}
