/*--------------------------------------------------------------------------
  LNTHETA_C.C: Guts of LNTHETA calculation.

  Log(Theta) function for BNLG and BLSG specifications.
  
  SYNTAX:
   double lntheta_c(double s, double a, double b, 
                                       int tau, int m, int n, double *c)

  Returns log(theta) of the same dimensions as s.

  Requires header file nulambda.h.
--------------------------------------------------------------------------*/
#include <math.h>
#include "nulambda.h"
#define accuracy 1e-12
#define limaccuracy 1e-7
#define lnbetafunc(a,b) (lgamma(a)+lgamma(b)-lgamma(a+b))
#define maxterms 50

/* Compuatationally expensive hypergeometric1F1 routine */
double hyperg1F1(double a, double b, double z) {
  int j=0;
  double r=1, M=1;
  while ((r/M)>accuracy) {
    j++;
    r*=(a+j-1)*z/(j*(b+j-1));
    M+=r;
    /*    printf("r=%le, M=%le \n",r,M); */
  }
  return(M);
}

/* function h(a,b,z) for limiting form */
double limitfactor(double a, double b, double z) {
  int j=0;
  double r=1.0, h=1.0, sm=0;
  while ((abs(r/h)>limaccuracy) && (sm<1)) {
    j++;
    sm=(b-a+j-1)*(j-a)/(j*z);
    r*=sm;
    h+=r;
  }
  return(h);
}

/* Theta function */
double lntheta_c(double s, double a, double b, int tau, int m, int n, double *c) {
 int j;
 double result,atau,batau,z,HH,rh=1,termfactor,lnapprox;
 double tally,M,rj=1;
 atau=a+tau*(n-m);
 batau=atau+b;
 z=n*tau*s;
 /* Decision to use asymptotic form depends on the number of terms in
    hypergeometric series expansion before terms start getting smaller.  */
 termfactor=(b+maxterms-1)*z/(maxterms*(batau+maxterms-1));
 /*   printf("termfactor=%f \n",termfactor); */
 lnapprox=z-(b-a)*log(z)+lgamma(b)-lgamma(a);
 if ((termfactor < 1) || (lnapprox < 600)) {
   M=hyperg1F1(b,batau,z);
   /*   printf("s=%f z=%f b=%f batau=%f M1=%e \n",s,z,b,batau,M); */
   tally=(*c)*M;
   for (j=1;j<m*(tau-1)+1;j++) {
     rj*=tau*s*(atau+j-1)/(batau+j-1);
     M=hyperg1F1(b,batau+j,z);
     tally+=(*(c+j))*rj*M;
   }
   result=log(tally)+lnbetafunc(atau,b);
 }
 else {
   /* use limiting form */
   HH=(*c)*limitfactor(b,batau,z);
   for (j=1;j<m*(tau-1)+1;j++) {
     rh*=(atau+j-1)/n;
     HH+=(*(c+j))*rh*limitfactor(b,batau+j,z);;
   }
   result=log(HH)+lgamma(atau)+z-atau*log(z);
 }
 return(result);
}

